@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * Author: Alexander Mikhaylenko <alexander.mikhaylenko@puri.sm>
+ * Author: Alice Mikhaylenko <alice.mikhaylenko@puri.sm>
  */
 
 #include "config.h"
@@ -25,7 +25,7 @@
  * </picture>
  *
  * The `AdwTabBar` widget is a tab bar that can be used with conjunction with
- * `AdwTabView`.
+ * `AdwTabView`. It is typically used as a top bar within [class@ToolbarView].
  *
  * `AdwTabBar` can autohide and can optionally contain action widgets on both
  * sides of the tabs.
@@ -37,6 +37,18 @@
  * ## CSS nodes
  *
  * `AdwTabBar` has a single CSS node with name `tabbar`.
+ *
+ * ## Style classes
+ *
+ * By default `AdwTabBar` look like a part of an `AdwHeaderBar` and is intended
+ * to be used directly attached to one or used as a [class@ToolbarView] toolbar.
+ * The [`.inline`](style-classes.html#inline) style class removes its background,
+ * so that it can be used in different contexts instead.
+ *
+ * <picture>
+ *   <source srcset="tab-bar-inline-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img src="tab-bar-inline.png" alt="tab-bar-inline">
+ * </picture>
  */
 
 struct _AdwTabBar
@@ -56,6 +68,8 @@ struct _AdwTabBar
   AdwTabView *view;
   gboolean autohide;
 
+  GdkDragAction extra_drag_preferred_action;
+
   gboolean is_overflowing;
   gboolean resize_frozen;
 };
@@ -76,6 +90,7 @@ enum {
   PROP_INVERTED,
   PROP_IS_OVERFLOWING,
   PROP_EXTRA_DRAG_PRELOAD,
+  PROP_EXTRA_DRAG_PREFERRED_ACTION,
   LAST_PROP
 };
 
@@ -239,14 +254,26 @@ stop_kinetic_scrolling_cb (GtkScrolledWindow *scrolled_window)
   gtk_scrolled_window_set_kinetic_scrolling (scrolled_window, TRUE);
 }
 
+static void
+set_extra_drag_preferred_action (AdwTabBar    *self,
+                                 GdkDragAction preferred_action)
+{
+  self->extra_drag_preferred_action = preferred_action;
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_EXTRA_DRAG_PREFERRED_ACTION]);
+}
+
 static gboolean
-extra_drag_drop_cb (AdwTabBar  *self,
-                    AdwTabPage *page,
-                    GValue     *value)
+extra_drag_drop_cb (AdwTabBar    *self,
+                    AdwTabPage   *page,
+                    GValue       *value,
+                    GdkDragAction preferred_action)
 {
   gboolean ret = GDK_EVENT_PROPAGATE;
 
+  set_extra_drag_preferred_action (self, preferred_action);
   g_signal_emit (self, signals[SIGNAL_EXTRA_DRAG_DROP], 0, page, value, &ret);
+  set_extra_drag_preferred_action (self, 0);
 
   return ret;
 }
@@ -365,6 +392,10 @@ adw_tab_bar_get_property (GObject    *object,
     g_value_set_boolean (value, adw_tab_bar_get_is_overflowing (self));
     break;
 
+  case PROP_EXTRA_DRAG_PREFERRED_ACTION:
+    g_value_set_flags (value, adw_tab_bar_get_extra_drag_preferred_action (self));
+    break;
+
   case PROP_EXTRA_DRAG_PRELOAD:
     g_value_set_boolean (value, adw_tab_bar_get_extra_drag_preload (self));
     break;
@@ -430,7 +461,7 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
   widget_class->compute_expand = adw_widget_compute_expand;
 
   /**
-   * AdwTabBar:view: (attributes org.gtk.Property.get=adw_tab_bar_get_view org.gtk.Property.set=adw_tab_bar_set_view)
+   * AdwTabBar:view:
    *
    * The tab view the tab bar controls.
    */
@@ -440,7 +471,7 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabBar:start-action-widget: (attributes org.gtk.Property.get=adw_tab_bar_get_start_action_widget org.gtk.Property.set=adw_tab_bar_set_start_action_widget)
+   * AdwTabBar:start-action-widget:
    *
    * The widget shown before the tabs.
    */
@@ -450,7 +481,7 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabBar:end-action-widget: (attributes org.gtk.Property.get=adw_tab_bar_get_end_action_widget org.gtk.Property.set=adw_tab_bar_set_end_action_widget)
+   * AdwTabBar:end-action-widget:
    *
    * The widget shown after the tabs.
    */
@@ -460,7 +491,7 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabBar:autohide: (attributes org.gtk.Property.get=adw_tab_bar_get_autohide org.gtk.Property.set=adw_tab_bar_set_autohide)
+   * AdwTabBar:autohide:
    *
    * Whether the tabs automatically hide.
    *
@@ -475,7 +506,7 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabBar:tabs-revealed: (attributes org.gtk.Property.get=adw_tab_bar_get_tabs_revealed)
+   * AdwTabBar:tabs-revealed:
    *
    * Whether the tabs are currently revealed.
    *
@@ -487,7 +518,7 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwTabBar:expand-tabs: (attributes org.gtk.Property.get=adw_tab_bar_get_expand_tabs org.gtk.Property.set=adw_tab_bar_set_expand_tabs)
+   * AdwTabBar:expand-tabs:
    *
    * Whether tabs expand to full width.
    *
@@ -500,7 +531,7 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabBar:inverted: (attributes org.gtk.Property.get=adw_tab_bar_get_inverted org.gtk.Property.set=adw_tab_bar_set_inverted)
+   * AdwTabBar:inverted:
    *
    * Whether tabs use inverted layout.
    *
@@ -513,7 +544,7 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabBar:is-overflowing: (attributes org.gtk.Property.get=adw_tab_bar_get_is_overflowing)
+   * AdwTabBar:is-overflowing:
    *
    * Whether the tab bar is overflowing.
    *
@@ -525,7 +556,24 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwTabBar:extra-drag-preload: (attributes org.gtk.Property.get=adw_tab_bar_get_extra_drag_preload org.gtk.Property.set=adw_tab_bar_set_extra_drag_preload)
+   * AdwTabBar:extra-drag-preferred-action:
+   *
+   * The unique action on the `current-drop` of the
+   * [signal@TabBar::extra-drag-drop].
+   *
+   * This property should only be used during a [signal@TabBar::extra-drag-drop]
+   * and is always a subset of what was originally passed to
+   * [method@TabBar.setup_extra_drop_target].
+   *
+   * Since: 1.4
+   */
+  props[PROP_EXTRA_DRAG_PREFERRED_ACTION] =
+    g_param_spec_flags ("extra-drag-preferred-action", NULL, NULL,
+                       GDK_TYPE_DRAG_ACTION, 0,
+                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwTabBar:extra-drag-preload:
    *
    * Whether the drop data should be preloaded on hover.
    *
@@ -710,7 +758,7 @@ adw_tab_bar_new (void)
 }
 
 /**
- * adw_tab_bar_get_view: (attributes org.gtk.Method.get_property=view)
+ * adw_tab_bar_get_view:
  * @self: a tab bar
  *
  * Gets the tab view @self controls.
@@ -726,7 +774,7 @@ adw_tab_bar_get_view (AdwTabBar *self)
 }
 
 /**
- * adw_tab_bar_set_view: (attributes org.gtk.Method.set_property=view)
+ * adw_tab_bar_set_view:
  * @self: a tab bar
  * @view: (nullable): a tab view
  *
@@ -802,7 +850,7 @@ adw_tab_bar_set_view (AdwTabBar  *self,
 }
 
 /**
- * adw_tab_bar_get_start_action_widget: (attributes org.gtk.Method.get_property=start-action-widget)
+ * adw_tab_bar_get_start_action_widget:
  * @self: a tab bar
  *
  * Gets the widget shown before the tabs.
@@ -818,7 +866,7 @@ adw_tab_bar_get_start_action_widget (AdwTabBar *self)
 }
 
 /**
- * adw_tab_bar_set_start_action_widget: (attributes org.gtk.Method.set_property=start-action-widget)
+ * adw_tab_bar_set_start_action_widget:
  * @self: a tab bar
  * @widget: (transfer none) (nullable): the widget to show before the tabs
  *
@@ -845,7 +893,7 @@ adw_tab_bar_set_start_action_widget (AdwTabBar *self,
 }
 
 /**
- * adw_tab_bar_get_end_action_widget: (attributes org.gtk.Method.get_property=end-action-widget)
+ * adw_tab_bar_get_end_action_widget:
  * @self: a tab bar
  *
  * Gets the widget shown after the tabs.
@@ -861,7 +909,7 @@ adw_tab_bar_get_end_action_widget (AdwTabBar *self)
 }
 
 /**
- * adw_tab_bar_set_end_action_widget: (attributes org.gtk.Method.set_property=end-action-widget)
+ * adw_tab_bar_set_end_action_widget:
  * @self: a tab bar
  * @widget: (transfer none) (nullable): the widget to show after the tabs
  *
@@ -888,7 +936,7 @@ adw_tab_bar_set_end_action_widget (AdwTabBar *self,
 }
 
 /**
- * adw_tab_bar_get_autohide: (attributes org.gtk.Method.get_property=autohide)
+ * adw_tab_bar_get_autohide:
  * @self: a tab bar
  *
  * Gets whether the tabs automatically hide.
@@ -904,7 +952,7 @@ adw_tab_bar_get_autohide (AdwTabBar *self)
 }
 
 /**
- * adw_tab_bar_set_autohide: (attributes org.gtk.Method.set_property=autohide)
+ * adw_tab_bar_set_autohide:
  * @self: a tab bar
  * @autohide: whether the tabs automatically hide
  *
@@ -934,7 +982,7 @@ adw_tab_bar_set_autohide (AdwTabBar *self,
 }
 
 /**
- * adw_tab_bar_get_tabs_revealed: (attributes org.gtk.Method.get_property=tabs-revealed)
+ * adw_tab_bar_get_tabs_revealed:
  * @self: a tab bar
  *
  * Gets whether the tabs are currently revealed.
@@ -952,7 +1000,7 @@ adw_tab_bar_get_tabs_revealed (AdwTabBar *self)
 }
 
 /**
- * adw_tab_bar_get_expand_tabs: (attributes org.gtk.Method.get_property=expand-tabs)
+ * adw_tab_bar_get_expand_tabs:
  * @self: a tab bar
  *
  * Gets whether tabs expand to full width.
@@ -968,7 +1016,7 @@ adw_tab_bar_get_expand_tabs (AdwTabBar *self)
 }
 
 /**
- * adw_tab_bar_set_expand_tabs: (attributes org.gtk.Method.set_property=expand-tabs)
+ * adw_tab_bar_set_expand_tabs:
  * @self: a tab bar
  * @expand_tabs: whether to expand tabs
  *
@@ -994,7 +1042,7 @@ adw_tab_bar_set_expand_tabs (AdwTabBar *self,
 }
 
 /**
- * adw_tab_bar_get_inverted: (attributes org.gtk.Method.get_property=inverted)
+ * adw_tab_bar_get_inverted:
  * @self: a tab bar
  *
  * Gets whether tabs use inverted layout.
@@ -1010,7 +1058,7 @@ adw_tab_bar_get_inverted (AdwTabBar *self)
 }
 
 /**
- * adw_tab_bar_set_inverted: (attributes org.gtk.Method.set_property=inverted)
+ * adw_tab_bar_set_inverted:
  * @self: a tab bar
  * @inverted: whether tabs use inverted layout
  *
@@ -1068,6 +1116,24 @@ adw_tab_bar_setup_extra_drop_target (AdwTabBar     *self,
   adw_tab_box_setup_extra_drop_target (self->pinned_box, actions, types, n_types);
 }
 
+/**
+ * adw_tab_bar_get_extra_drag_preferred_action:
+ * @self: a tab bar
+ *
+ * Gets the current action during a drop on the extra_drop_target.
+ *
+ * Returns: the drag action of the current drop.
+ *
+ * Since: 1.4
+ */
+GdkDragAction
+adw_tab_bar_get_extra_drag_preferred_action (AdwTabBar *self)
+{
+  g_return_val_if_fail (ADW_IS_TAB_BAR (self), 0);
+
+  return self->extra_drag_preferred_action;
+}
+
 /**
  * adw_tab_bar_get_extra_drag_preload:
  * @self: a tab bar
@@ -1113,7 +1179,7 @@ adw_tab_bar_set_extra_drag_preload (AdwTabBar *self,
 }
 
 /**
- * adw_tab_bar_get_is_overflowing: (attributes org.gtk.Method.get_property=is-overflowing)
+ * adw_tab_bar_get_is_overflowing:
  * @self: a tab bar
  *
  * Gets whether @self is overflowing.
